/***********************************************************************
**
** WinSysInfo.dll
**
** August 30, 1998
**
** by Ryan J. Stradling - Real3D
**
** Based on code written for EnvWin.c, EnvWin.h written for GLPerf
**
** This dll will report system information to the user.
** The setup is that all the functions return one of the error results
** stated below.  
** All of the functions assume that the user has allocated memory for 
** the variables passed into the functions.  All the string functions
** are passed in the size of the string. An error is returned from these 
** functions if the string passed in is not big enough to hold the data.
**
** NOTES:
**  Right now this DLL does not include generic support for reporting
**  of the display size, the window size, or OpenGL Buffer information.
**
*************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <math.h>
#include <malloc.h>
#include <process.h>
#include <gl\glaux.h>


#include "Env.h"


/* ERROR RESULTS */
#define E_NONE 1                    /* No error occurred 
				     */
#define E_BADSTRINGSIZE 0           /* The string size passed in was not 
					large enough 
				     */
#define E_BADRETURNFUNCTIONCALL -1  /* A function was called that returned 
					an error; Not a fatal error
				     */


/******************************************************************************
 *
 * StringSearch - search for pattern in string
 *
 * Description:
 *  StringSearch returns a pointer to the first occurance of pattern found in
 *  the subject or NULL if the pattern is not found. It implements the
 *  Knuth-Morris-Pratt pattern matching algorithm. If m is the length of the
 *  pattern and n is the length of the subject, the complexity of the KMP
 *  algorithm is (m+n), much better than the (m*n) complexity of the naive
 *  nested loop algorithm. - John Dennis
 *
 * Returns:
 *  pointer to the first occurance of pattern found the subject or NULL
 *
 * Side Effects:
 *  None
 *
 * Errors:
 *  None
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Thu Aug  4 15:49:59 1994
 *    Initial Release
 *
 *****************************************************************************/
char *
StringSearch(char *subject, char *pattern)
{
#define MAX_FLINK (256)
  int *flink, *dynamicFlink = NULL, staticFlink[MAX_FLINK];
  int patternLen = strlen(pattern);
  int subjectLen = strlen(subject);
  int found = 0;
  int i,j;

  if (patternLen > MAX_FLINK) {      /* -1 for NULL terminator */
    dynamicFlink = (int*) malloc(patternLen * sizeof(int));
    if (dynamicFlink == NULL) {
      fprintf(stderr, "malloc failure, line %d, file: %s, exiting...\n",
	      __LINE__, __FILE__);
      exit(1);
    }
    flink = dynamicFlink;
  }
  else {
    flink = staticFlink;
  }


  /* Step 1: Constuct Flowchart */
  flink[0] = -1;                       /* -1 == read next char */
  for(i = 1; i < patternLen; i++) {
    j = flink[i-1];
    while((j >= 0) && (pattern[j] != pattern[i-1])) {
      j = flink[j];
    }
    flink[i] = j+1;
  }

  /* Step 2: Scan Algorithm */
  for (i = j = 0; i < subjectLen; i++, j++) {
    while((j >= 0) && (pattern[j] != subject[i])) {
      j = flink[j];
    }
    if (j == patternLen-1) {
      found = 1;
      goto exit;
    }
  }

 exit:
  if (dynamicFlink != NULL) free(dynamicFlink);
  if (!found)
    return(NULL);
  else
    return(&subject[i-patternLen+1]);
#undef MAX_FLINK
}


/******************************************************************************
 *
 * GetShortVendorName - return short vendor string
 *
 * Description:
 *  some vendor strings are verbose. This function will return a shortest
 *  vendor name it can given an arbitrary vendor string. The returned string
 *  is allocated with malloc, it should be freed when no longer in use. The
 *  input string is not freed or modified by this function.
 *
 * Returns:
 *  pointer to allocated string
 *
 * Side Effects:
 *  string allocation, string should be freed when no longer needed.
 *
 * Errors:
 *  no errors, if function fails the string "unknown" is returned
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *****************************************************************************/
char *
GetShortVendorName(char *input)
{
  int i;
  char *s1 = NULL;              /* s1 is temp work string */
  char *s2 = NULL;              /* s2 is string to return */

  if (!input)
	  return NULL;
  /* duplicate input string so that we can modify it */
  s1 = strdup(input);
  /* upcase string */
  for (i = 0; s1[i]; i++)
    if (islower(s1[i])) s1[i] = toupper(s1[i]);

  /* return a short name if possible, some vendor names are verbose */
  if (StringSearch(s1, "DEC") ||
      StringSearch(s1, "DECWINDOWS") ||
      StringSearch(s1, "Digital Equipment Corporation") ||
      StringSearch(s1, "DigitalEquipmentCorporation"))
    s2 = strdup("DEC");
  else if (StringSearch(s1, "SGI") ||
	   StringSearch(s1, "SILCON GRAPHICS"))
    s2 = strdup("SGI");
  else if (StringSearch(s1, "IBM") ||
	   StringSearch(s1, "INTERNATIONAL BUSINESS MACHINES"))
    s2 = strdup("IBM");
  else
    s2 = strdup(s1);

  free(s1);
  return(s2);
}


/******************************************************************************
 *
 * GetHostMemorySize - gets megabytes of memory installed on host platform
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   MemSize     = passes back the HostMemory size
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Thu Aug  4 15:55:19 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostMemorySize(int *MemSize)
{
/* RESERVED_MEMORY accounts for memory reserved for extended bios data area
 * that is not included in the total physical memory returned from the
 * GlobalMemoryStatus call on x86.
 */
#define RESERVED_MEMORY  (500000)
    
  MEMORYSTATUS MemoryStatus;

  MemoryStatus.dwLength = sizeof(MEMORYSTATUS);
  GlobalMemoryStatus(&MemoryStatus);
  *MemSize =  (MemoryStatus.dwTotalPhys + RESERVED_MEMORY) / 1048576;
  return E_NONE;
}
/******************************************************************************
 *
 * GetHostModel - gets the host model
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the host model
 *   SizeVers   = size of the allocated variable
 *
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostModel(char *string, unsigned int size)
{
  
HKEY hkGlobal;
  DWORD dwType,chData=256;
  BYTE DataBuffer[256];
  int  NT = 0;
  DWORD version;

  /* Added by Stradling @ real3D to test for windows95 or NT os */
  /* To get rid of searching for an invalid key in the registry */
  /* in Winows 95 and displaying an annoying message to the screen */
  version = GetVersion();

  if (version & 0x80000000)
    NT = 0;  
  else
    NT = 1;  

  
  if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE,"HARDWARE\\DESCRIPTION\\System",
	    0, KEY_READ, &hkGlobal) != ERROR_SUCCESS)
  {
    fprintf(stderr, "could not open registry key \
HKEY_LOCAL_MACHINE\\HARDWARE\\DESCRIPTION\\System\n");
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }

  if ( RegQueryValueEx(hkGlobal, "Identifier", NULL, &dwType,
	    DataBuffer, &chData) != ERROR_SUCCESS)
  {
	if ( NT ) /* If NT then registry should have the info */
    {
		fprintf(stderr, "could not read registry value \
		HKEY_LOCAL_MACHINE\\HARDWARE\\DESCRIPTION\\System\\Identifier\n");
	}
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }

  if (strlen(DataBuffer) > size)
      return E_BADSTRINGSIZE;
  strcpy(string, (char*)DataBuffer);
  return E_NONE;
}


/******************************************************************************
 *
 * GetHostVendor - gets the system vendor
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the Host Vendor
 *   SizeVers   = size of the allocated variable
 *
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostVendor(char *string, unsigned int size)
{
  /* See if it's a DEC system */
  char *str;
  unsigned int lSize = MAX_PATH;

  str = malloc(sizeof(char)*lSize);
  GetHostModel(str, MAX_PATH);

  if ( strstr(str,"DEC") == str)
  {
    free(str);
    if (strlen(string) > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "DEC");
  }
  else
  {
    free(str);
    if (strlen(string) > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
  }
  return E_NONE;
}




/******************************************************************************
 *
 * GetHostCPU - gets the host CPU
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the host CPU
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostCPU(char *string, unsigned int size)
{
  char  szRegInfoValue[1024];
  HKEY  hkey;
  DWORD rc;
  DWORD cb;

  if (!(rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
			  "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",
			  0, KEY_READ, &hkey))) {
      cb = sizeof(szRegInfoValue);
      rc = RegQueryValueEx(hkey, "Identifier", 0, 0, (LPBYTE)szRegInfoValue, &cb);
      RegCloseKey(hkey);
  }

  if (!rc)
  {
      if (strlen(szRegInfoValue) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, szRegInfoValue);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
  }
  return E_NONE;
}

/******************************************************************************
 *
 * GetHostPageSize - gets the PageSize
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *  PageSize = Page Size
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Feb 27 14:18:41 EST 1995
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostPageSize(LPDWORD PageSize)
{
    SYSTEM_INFO Sys;
    GetSystemInfo(&Sys);
    *PageSize = Sys.dwPageSize;
    return E_NONE;
}

/******************************************************************************
 *
 * GetHostCPUCount - gets number of host CPU's
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the host CPU Count
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Feb 27 14:18:41 EST 1995
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostCPUCount(char *string, unsigned int size)
{
  SYSTEM_INFO Sys;
  char tmp[5];
  
  GetSystemInfo(&Sys);
  if (strlen(string) > size)
      return E_BADSTRINGSIZE;
  itoa(Sys.dwNumberOfProcessors, tmp, 10);
  strcpy(string, tmp);
  return E_NONE;
}

/******************************************************************************
 *
 * GetHostPrimaryCacheSize - gets the host's primary cache (KB)
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the PrimaryCache Size
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Feb 27 14:18:41 EST 1995
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short 
GetHostPrimaryCacheSize(char *string, unsigned int size)
{
  if (strlen("unknown") > size)
      return E_BADSTRINGSIZE;
  strcpy(string, "unknown");
  return E_NONE;
}

/******************************************************************************
 *
 * GetHostSecondaryCacheSize - gets the host's secondary cache (KB)
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the Secondary Cache Size
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Feb 27 14:18:41 EST 1995
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostSecondaryCacheSize(char *string, unsigned int size)
{
  if (strlen("unknown") > size)
      return E_BADSTRINGSIZE;
  strcpy(string, "unknown");
  return E_NONE;
}

/******************************************************************************
 *
 * GetWindowSystem - gets the windowing system
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the window system
 *   SizeVers   = size of the allocated variable
 *
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetWindowSystem(char *string, unsigned int size)
{
  if (strlen("Win32") > size)
      return E_BADSTRINGSIZE;
  strcpy(string, "Win32");
  return E_NONE;
}

/******************************************************************************
 *
 * GetDriverVersion - gets the graphics driver
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the Driver version
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Feb 27 15:00:18 EST 1995
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetDriverVersion(char *string, unsigned int size)
{
  if (strlen("unknown") > size)
      return E_BADSTRINGSIZE;
  strcpy(string, "unknown");
  return E_NONE;
}

/******************************************************************************
 *
 * GetHostOperatingSystem - gets the host CPU operating system
 *
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the Operating System
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostOperatingSystem(char *string, unsigned int size)
{
  OSVERSIONINFO Win32VersionInformation;

  Win32VersionInformation.dwOSVersionInfoSize = sizeof(Win32VersionInformation);
  GetVersionEx(&Win32VersionInformation);

  if (Win32VersionInformation.dwPlatformId == VER_PLATFORM_WIN32s) {
    if (strlen("Microsoft Win32s with Windows 3.1") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "Microsoft Win32s with Windows 3.1");
  }
  else if (Win32VersionInformation.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) {
    if (strlen("Microsoft Windows 95") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "Microsoft Windows 95");
  }
  else if (Win32VersionInformation.dwPlatformId == VER_PLATFORM_WIN32_NT) {
    if (strlen("Microsoft Windows NT") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "Microsoft Windows NT");
  }
  else {
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
  }
  return E_NONE;
}


/******************************************************************************
 *
 * GetHostOperatingSystemRelease - gets operating system release
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the operating system release
 *   SizeVers   = size of the allocated variable
 *
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostOperatingSystemRelease(char *string, unsigned int size)
{
  CHAR szRelease[256];
  CHAR szTitle[128];
  CHAR szDebug[16];
  OSVERSIONINFO Win32VersionInformation;

  Win32VersionInformation.dwOSVersionInfoSize = sizeof(Win32VersionInformation);
  if (GetVersionEx(&Win32VersionInformation)) {
    szTitle[0] = 0;
    if (Win32VersionInformation.szCSDVersion[0] != 0) {
	sprintf(szTitle, ": %s", Win32VersionInformation.szCSDVersion);
    }
    szDebug[0] = 0;
    if (GetSystemMetrics(SM_DEBUG)) {
	sprintf(szDebug, " (Debug)");
    }
    sprintf(szRelease, "Version %d.%d (Build %d%s)%s",
	    Win32VersionInformation.dwMajorVersion,
	    Win32VersionInformation.dwMinorVersion,
	    Win32VersionInformation.dwBuildNumber,
	    (LPSTR)szTitle,
	    (LPSTR)szDebug
	   );
    if (strlen(szRelease) > size)
	return E_BADSTRINGSIZE;
    strcpy(string, szRelease);
  }
  else {
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}


/******************************************************************************
 *
 * GetHostName - get the host's name
 *
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the host's name
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetHostName(char *string, unsigned int size)
{
  DWORD cchBuffer = 128;
  BYTE szName[128];

  if ( ! GetComputerName((char*)szName,&cchBuffer)) {
    if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
  }
  else {
      if (strlen(szName) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, szName);
      return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}



/******************************************************************************
 *
 * GetSDate - get the current month, day, year
 *
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the date in MM/DD/YYYY
 *   SizeVers   = size of the allocated variable
 *
 *
 * Errors:
 *  return NULL on failure
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Wed Aug 17 13:08:36 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling Date: Monday April 20 10:40 1998
 *    Made a new function GetSDate based on author's above GetDateTime() 
 *
 *****************************************************************************/

short 
GetSDate(char *string, unsigned int size)
{
  time_t timeT;
  struct tm *timeTm;
  char date[18];
  char m[5];
  char d[5];
  char y[5];
  int month, day, year;

  time(&timeT);
  timeTm = localtime(&timeT);
  if (timeTm == NULL) {
    fprintf(stderr, "Error calling localtime: %d\n", errno);
    return E_BADRETURNFUNCTIONCALL;
  }
  month  = timeTm->tm_mon + 1;
  day    = timeTm->tm_mday;
  year   = timeTm->tm_year + 1900;
  itoa(month, m, 10);
  itoa(day, d, 10);
  itoa(year, y, 10);
  date[0] = '\0';
  if (month < 10)
      strcpy(date, "0");
  strcat(date, m);
  strcat(date, "\\");
  if (day<10)
      strcat(date, "0");
  strcat(date, d);
  strcat(date, "\\");
  strcat(date, y);

  if (strlen(date) > size)
      return E_BADSTRINGSIZE;
  strcpy(string, date);
  return E_NONE;
}
/******************************************************************************
 *
 * GetSTime - get the current hour, minute
 *
 * Description:
 *  Get the current hour, minute. 
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the time in (HH:MM)
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Wed Aug 17 13:08:36 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling Date: Monday April 20 10:40 1998
 *    Made a new function GetSTime based on author's above GetDateTime() 
 *
 *****************************************************************************/
short
GetSTime(char *string, unsigned int size)
{
  time_t timeT;
  struct tm *timeTm;
  char tme[10];
  char h[3];
  char m[3];
  int hour, minute;

  time(&timeT);
  timeTm = localtime(&timeT);
  if (timeTm == NULL) {
    fprintf(stderr, "Error calling localtime: %d\n", errno);
    return E_BADRETURNFUNCTIONCALL;
  }
  hour   = timeTm->tm_hour;
  minute = timeTm->tm_min;
  itoa(hour, h, 10);
  itoa(minute, m, 10);
  strcpy(tme, h);
  strcat(tme, ":");
  if (minute <10)
      strcat(tme, "0");
  strcat(tme, m);
  if (strlen(tme) > size)
      return E_BADSTRINGSIZE;
  strcpy(string, tme);
  return E_NONE;
}


/******************************************************************************
 *
 * GetOpenGLClientVendor - gets the client library vendor
 *
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the GLClient vendor
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetOpenGLClientVendor(char *string, unsigned int size)
{
  char *pstr;
  char *pstr2;
  if (pstr = (char *)glGetString(GL_VENDOR)) {
    pstr2 = GetShortVendorName(pstr);
    if (strlen(pstr2) > size)
	return E_BADSTRINGSIZE;
    strcpy(string, pstr2);
    free(pstr2);
  }
  else {
    if (strlen("unknown") > size)
	E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}


/******************************************************************************
 *
 * GetOpenGLClientVersion - return string naming the client library version
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the GLClient version
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetOpenGLClientVersion(char *string, unsigned int size)
{
  char *pstr;
  if (pstr = (char *)glGetString(GL_VERSION)) {
    if (strlen(pstr) > size )
	return E_BADSTRINGSIZE;
    strcpy(string, pstr);
  }
  else {
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}


/******************************************************************************
 *
 * GetOpenGLClientExtensions - return string naming the client extensions
 *
 * Returns:
 *   E_NONE on success
 *   E_BADRETURNFUNCTIONCALL on success with ERRORS
 *   E_BADSTRINGSIZE when size of char* is not large enough
 *
 *
 * Parameters:
 *   string     = passes back the client extensions
 *   SizeVers   = size of the allocated variable
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Mon Aug  8 16:30:53 1994
 *    Initial Release
 *  Revision 1: Author: Ryan J. Stradling: 4/25/98
 *    Changed the code for the new DLL format
 *
 *****************************************************************************/
short
GetOpenGLClientExtensions(char *string, unsigned int size)
{
  char *pstr;
  if (pstr = (char *)glGetString(GL_EXTENSIONS)) {
    if (strlen(pstr) > size)
	return E_BADSTRINGSIZE;
    strcpy(string, pstr);
  }
  else {
    if (strlen("unknown") > size)
	return E_BADSTRINGSIZE;
    strcpy(string, "unknown");
    return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}



/******************************************************************************
 *
 * GetOpenGLBufferInfo - inquire the OpenGL buffer configuration
 *
 * Description:
 *  Given a specfic rendering target and visual description, return the OpenGL
 *  buffer configuration. This information includes:
 *
 * Returns:
 *  0 for success, error code otherwise
 *
 * Side Effects:
 *  None
 *
 * Errors:
 *  None
 *
 * Revision History:
 *  Revision 0: Author: John R. Dennis Date: Thu Aug 18 10:18:45 1994
 *    Initial Release
 *
 *****************************************************************************/

 int
GetOpenGLBufferConfig(HDC hdc, EnvironmentInfo *info)
{
  PIXELFORMATDESCRIPTOR pfd;
  BufferConfig *config = &info->bufConfig;

  config->ipfd = GetPixelFormat(hdc);

  DescribePixelFormat(hdc, config->ipfd, sizeof(pfd), &pfd);

  config->doubleBuffer = pfd.dwFlags & PFD_DOUBLEBUFFER;
  config->stereo = pfd.dwFlags & PFD_STEREO;
  config->rgba = pfd.iPixelType == PFD_TYPE_RGBA;

  if (config->rgba) {
    config->indexSize = -1;

    config->redSize   = pfd.cRedBits;
    config->greenSize = pfd.cGreenBits;
    config->blueSize  = pfd.cBlueBits;
    config->alphaSize = pfd.cAlphaBits;

    config->accumRedSize   = pfd.cAccumRedBits;
    config->accumGreenSize = pfd.cAccumGreenBits;
    config->accumBlueSize  = pfd.cAccumBlueBits;
    config->accumAlphaSize = pfd.cAccumAlphaBits;
  }
  else {
    config->indexSize = pfd.cColorBits;

    config->redSize   = -1;
    config->greenSize = -1;
    config->blueSize  = -1;
    config->alphaSize = -1;

    config->accumRedSize   = -1;
    config->accumGreenSize = -1;
    config->accumBlueSize  = -1;
    config->accumAlphaSize = -1;
  }

  config->depthSize   = pfd.cDepthBits;
  config->stencilSize = pfd.cStencilBits;
  config->auxBuffers  = pfd.cAuxBuffers;

  config->level = pfd.iLayerType;

  return E_NONE;
}



/******************************************************************************
*
* GetGLVendor - Gets the GL Vendor name
*
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GL Version name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/
short
GetGLVendor(char *string, unsigned int size) 
{
  char *str = GetShortVendorName((char *) glGetString(GL_VENDOR));
 
  if (str) {
     if (strlen(str) > size)
	return E_BADSTRINGSIZE;
     strcpy(string, str);
     free(str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}

/******************************************************************************
*
* GetGLVersion - Gets the GL Version name
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GL Version name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/

short
GetGLVersion(char *string, unsigned int size)
{
  char *str;

  str = (char *) glGetString(GL_VERSION);
  if (str) {
      if (strlen(str) > size)
	  E_BADSTRINGSIZE;
      strcpy(string, str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}  

/******************************************************************************
*
* GetGLRenderer - Gets the GLRenderer name
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GLRenderer name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/

short
GetGLRenderer(char *string, unsigned int size)
{
  char *str = (char *) glGetString(GL_RENDERER);
  if (str) {
      if (strlen(str) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
}

/******************************************************************************
*
* GetGLExtensions - Gets the GLU Extensions
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GLExtensions name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/
short
GetGLExtensions(char *string, unsigned int size)
{
  char *str = (char *) glGetString(GL_EXTENSIONS);
  if (str) {
      if (strlen(str) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }
  return E_NONE;
} 

/******************************************************************************
*
* GetGLUVersion - Gets the GLU Version name
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GLU Version name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/
short
GetGLUVersion(char *string, unsigned int size)
{
  char *str;
#ifdef GLU_VERSION_1_1
  str = (char *)gluGetString(GLU_VERSION);
  if (str) {
      if (strlen(str) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }
#else
  if (strlen("unknown") > size)
       return E_BADSTRINGSIZE;
  strcpy(string, "unknown");
#endif
return E_NONE;
}

/******************************************************************************
*
* GetGLUExtensions - Gets the GLU Extensions
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   string     = passes back the GLU Extensions name
*   SizeVers   = size of the allocated variable
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/
short
GetGLUExtensions(char *string, unsigned int size)
{
  char *str;
#ifdef GLU_VERSION_1_1
  str = (char*)gluGetString(GLU_EXTENSIONS);
  if (str) {
      if (strlen(str) > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, str);
  }
  else {
      if (strlen("unknown") > size)
	  return E_BADSTRINGSIZE;
      strcpy(string, "unknown");
      return E_BADRETURNFUNCTIONCALL;
  }

#else
  if (strlen("unknown") > size)
     return E_BADSTRINGSIZE;
  strcpy(string, "unknown");
#endif
return E_NONE;
}

/******************************************************************************
*
* GetDirectRender - Gets DirectRender
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*    Render = 1 for true, 0 for fals
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/

short 
GetDirectRender(int *Render)
{
#if defined(WIN32)
  /*
   * There is no direct rendering at this time.
   */
  *Render = 0;
#else
  /*
   * Direct rendering is a "request", not a guarantee. We need to check the
   * direct render bit for each test because the GL context is only set up
   * in the above conditional block. If we didn't check it for each test
   * then the value for direct rendering store in the test descriptor would
   * be the "request" value, not the actual value.
   */

  windType = auxGetDisplayMode();
  if (windType & AUX_INDIRECT)
    *Render = 0;
  else
    *Render = 1;
#endif
  return E_NONE;
}


/******************************************************************************
*
* GetHostDisplayDriver - Gets the name of the display driver
*
* Description:
*   Looks through the registry information to find the display driver
*   
* Returns:
*   E_NONE on success
*   E_BADRETURNFUNCTIONCALL on success with ERRORS
*   E_BADSTRINGSIZE when size of char* is not large enough
*
*
* Parameters:
*   winNT      = 1 if windows NT being used, 0 if 95
*   DriverName = passes back the DriverName
*   SizeName   = size of the allocated variable DriverName
*   DriverVers = passes back the DriverVersion
*   SizeVers   = size of the allocated variable SizeVers 
*
* Revision History:
*   Revision 0: Ryan J. Stradling : Wednesday April 22, 1998
*
*******************************************************************************/
short 
GetHostDisplayDriver( int winNT, char *DriverName, unsigned int SizeName, 
		     char *DriverVers, unsigned int SizeVers )
{
   HKEY hkGlobal, foundKey, tempKey;
   DWORD chData, dwType = 512;
   BYTE DataBuffer[512];
   char *place;
   LPSTR lptstrVffInfo;
   LPSTR lpVersion;
   UINT*    puVersionLen;
   HANDLE hMem;
   int bRetCode;
   char DLL[MAX_PATH];
   char RealDLLName[MAX_PATH+20];
   DWORD FileInfoSize, dwTemp;
   FILE *fp;
   char line[256];
   int found;
   BOOL StarFound;
   BYTE lpClass[512];
   DWORD lpcbClass = 512;
   DWORD numSubKeys = 0;
   DWORD numDisplayKeys = 0;
   DWORD index = 0;
   char lpName[5];
   DWORD size = 512;
   BYTE bBuffer[512];
   long HexNumber;
   char Lang_Charset_String[10], FinalString[10], VersionString[256];
   DEVMODE DevMode;

   if (winNT == 1) {
      RealDLLName[0] = '\0';

      /* Get the Display Driver Name */
      EnumDisplaySettings(NULL, 0, &DevMode);
      strcpy(RealDLLName, DevMode.dmDeviceName);
       FileInfoSize = GetFileVersionInfoSize(RealDLLName, &dwTemp);
       if (FileInfoSize) {
	   hMem = GlobalAlloc(GMEM_MOVEABLE, FileInfoSize);
		     lptstrVffInfo  = (char *)GlobalLock(hMem);
	   memset(lptstrVffInfo, '\0', (FileInfoSize*sizeof(char)));
	   GetFileVersionInfo(RealDLLName, dwTemp, FileInfoSize, lptstrVffInfo);

	  /* Get the character and language set */
	  puVersionLen = malloc(sizeof(UINT));
	  bRetCode = VerQueryValue((LPVOID)lptstrVffInfo,
	      TEXT("\\VarFileInfo\\Translation"), (LPVOID*)&lpVersion,
	      puVersionLen);


	  if (!bRetCode) {
	      if (puVersionLen)
		  free(puVersionLen);
	      if (hMem) GlobalFree(hMem);
	      if (strlen(DriverVers) > SizeVers) {
		   return E_BADSTRINGSIZE;
	      }
	      strcpy(DriverVers, "unknown");
		  return E_BADRETURNFUNCTIONCALL;
	  }

	  /* Convert the HEX number into a string */
	  MoveMemory(bBuffer, lpVersion, *puVersionLen);
	  HexNumber = bBuffer[2] + bBuffer[3] * 0x100 + 
	      bBuffer[0] * 0x10000 + bBuffer[1] * 0x1000000;
	  itoa(HexNumber, Lang_Charset_String, 16); 
	  memset(FinalString, '\0', sizeof(char)*10);

	  /* Add leading 0's */
	  index = 0;
	  while (index != (8-strlen(Lang_Charset_String))) {
	      FinalString[index] = '0';
	      index++;
	  }
	  strcat(FinalString, Lang_Charset_String);
	  strcpy(VersionString, "\\StringFileInfo\\");
	  strcat(VersionString, FinalString);
	  strcat(VersionString, "\\FileVersion");

	  /* Get the product version from the character set info */
	  lpVersion = NULL;
	  bRetCode = VerQueryValue((LPVOID)lptstrVffInfo,
	      VersionString,
	      (LPVOID*)&lpVersion,
	      puVersionLen);

	  if (bRetCode) {
	     if (strlen((char*)lpVersion) > SizeName) {
		 if (puVersionLen)
		     free(puVersionLen);
		 if (hMem)
		     GlobalFree(hMem);
		 return E_BADSTRINGSIZE;
	     }
	     else {
		strcpy(DriverVers, lpVersion);
	     }
	  }
	  else {
	     if (strlen("unknown") > SizeName) {
	     }
	     strcpy(DriverVers, "unknown");
	  }

	  /* New below */
	  strcpy(VersionString, "");
	  strcpy(VersionString, "\\StringFileInfo\\");
	  strcat(VersionString, FinalString);
	  strcat(VersionString, "\\FileDescription");
	  bRetCode = VerQueryValue((LPVOID)lptstrVffInfo,
	     VersionString,
	     (LPVOID*)&lpVersion,
	     puVersionLen);

	  if (puVersionLen)
	     free(puVersionLen);

	  if (bRetCode) {
	     if (strlen((char*)lpVersion) > SizeVers) {
		if (hMem)
		   GlobalFree(hMem);
		return E_BADSTRINGSIZE;
	     }
	     strcpy(DriverName, lpVersion);
	     if (hMem)
		GlobalFree(hMem);              
	  }
	  else {
	     if (strlen("unknown") > SizeVers) {
		if (hMem)
		   GlobalFree(hMem);              
		return E_BADSTRINGSIZE;
	     }
	     strcpy(DriverName, "unknown");
	     if (hMem)
		GlobalFree(hMem);  
	     return E_BADRETURNFUNCTIONCALL;
	  }
	   return E_NONE;
       }
       else {
	   if (strlen(DriverVers) > SizeVers) {
		return E_BADSTRINGSIZE;
	   }
	   strcpy(DriverVers, "unknown");
       }
    }

    else if (winNT == 0) {
       /* Use 95 search path */

	/* Search in the .ini file for the driver name */
	GetWindowsDirectory(DLL, MAX_PATH);
	strcpy(RealDLLName, DLL);
	strcat(RealDLLName, "\\system.ini");
	fp = fopen(RealDLLName, "r");
	if (!fp) {
	    if (strlen("unknown") > SizeName)
		return E_BADSTRINGSIZE;
	    strcpy(DriverName, "unknown");
	    if (strlen(DriverName) > SizeVers)
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
	    return E_BADRETURNFUNCTIONCALL;
	}
	found = 0;
	while ((fgets(line, 256, fp)) != NULL) {
	    if (strstr(line, "boot.description")) {
		found = 1;
		break;
	    }
	}
	if (!found) {
	    fclose(fp);
	    if (strlen("unknown") > SizeName)
		return E_BADSTRINGSIZE;
	    strcpy(DriverName, "unknown");
	    if (strlen(DriverName) > SizeVers)
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
	    return E_BADRETURNFUNCTIONCALL;
	}
	while ((fgets(line, 256, fp)) != NULL) {
	    if (strstr(line, "display.drv")) {
		found = 1;
		break;
	    }
	}
	if (!found) {
	    fclose(fp);
	    if (strlen("unknown") > SizeName)
		return E_BADSTRINGSIZE;
	    strcpy(DriverName, "unknown");
	    if (strlen(DriverName) > SizeVers)
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
	    return E_BADRETURNFUNCTIONCALL;
	}
	fclose(fp);
	place = strstr(line, "=");
	place++;
	if (strlen(place) > SizeName)
	    return E_BADSTRINGSIZE;
	/* Now we have the name of the driver to search through
	  the registry for */
	strcpy(DriverName, place);
	place = strchr(DriverName, '\n');
	*place = '\0';  
 

	/* Start looking through the registry */
	    if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE,
	    "System\\CurrentControlSet\\Services\\Class\\Display",
		0, KEY_READ, &hkGlobal) != ERROR_SUCCESS) {
	    if (strlen(DriverVers) > SizeVers);
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
		    return E_BADRETURNFUNCTIONCALL;
	    }
	if ( RegQueryInfoKey ( hkGlobal, lpClass, &lpcbClass, 0, &numSubKeys, NULL, NULL, NULL, NULL, NULL, NULL, NULL ) 
			    != ERROR_SUCCESS) {
	    if (strlen(DriverVers) > SizeVers);
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
		    return E_BADRETURNFUNCTIONCALL;
	    }
	memset (lpName, '\0', 5*sizeof(char) );

	StarFound = FALSE;
	chData = 512;
	    /* Found the driver in the registry */
	    for ( index = (numSubKeys-1); index != -1; index-- ) {
		    if (RegEnumKeyEx( hkGlobal, index, lpName, &size, 0, 0, 0, 0) != ERROR_NO_MORE_ITEMS ) {
		    if (RegOpenKeyEx(hkGlobal,lpName, 0, KEY_READ, &foundKey) == ERROR_SUCCESS) {
			    if ( (RegQueryValueEx(foundKey, "DriverDesc", NULL, &dwType, DataBuffer, &chData)) == ERROR_SUCCESS) {
				    if (strstr(DataBuffer, DriverName)) {
					    StarFound = TRUE;
					    break;
				    }
			    }
		    }
	    }
		    DataBuffer[0] = '\0';
		    chData = 512;
	    }
	if (!StarFound) {
	    if (strlen(DriverVers) > SizeVers);
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
		    return E_BADRETURNFUNCTIONCALL;
	}

	/* Driver was found !!! */
	    if ( RegOpenKeyEx(foundKey, "DEFAULT", 0, KEY_READ, &tempKey) != ERROR_SUCCESS ) {
	    if (strlen(DriverVers) > SizeVers);
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
		    return E_BADRETURNFUNCTIONCALL;
	    }
	chData = 512;
	DataBuffer[0] = '\0';
	if ( RegQueryValueEx(tempKey, "drv", NULL, &dwType,
	    DataBuffer, &chData) != ERROR_SUCCESS) {
	    if (strlen(DriverVers) > SizeVers);
		return E_BADSTRINGSIZE;
	    strcpy(DriverVers, "unknown");
		    return E_BADRETURNFUNCTIONCALL;
	}
	GetSystemDirectory(DLL, MAX_PATH);
	strcpy(RealDLLName, DLL);
	strcat(RealDLLName, "\\");
	strcat(RealDLLName, DataBuffer);
	/* Get the Driver ProductVersion Info from the DLL */
      FileInfoSize = GetFileVersionInfoSize(RealDLLName, &dwTemp);
	if (FileInfoSize) {
	   hMem = GlobalAlloc(GMEM_MOVEABLE, FileInfoSize);
		   lptstrVffInfo  = (char *)GlobalLock(hMem);
	   memset(lptstrVffInfo, '\0', (FileInfoSize*sizeof(char)));
	   GetFileVersionInfo(RealDLLName, dwTemp, FileInfoSize, lptstrVffInfo);
	   
	   /* Get the character and language set */
	   puVersionLen = malloc(sizeof(UINT));
	   bRetCode = VerQueryValue((LPVOID)lptstrVffInfo,
	       TEXT("\\VarFileInfo\\Translation"), (LPVOID*)&lpVersion,
	       puVersionLen);

	   if (!bRetCode) {
	       if (puVersionLen)
		   free(puVersionLen);
	       if (hMem) GlobalFree(hMem);
	       if (strlen(DriverVers) > SizeVers) {
		    return E_BADSTRINGSIZE;
	       }
	       strcpy(DriverVers, "unknown");
		   return E_BADRETURNFUNCTIONCALL;
	   }

	   /* Convert the HEX number into a string */
	   MoveMemory(bBuffer, lpVersion, *puVersionLen);
	   HexNumber = bBuffer[2] + bBuffer[3] * 0x100 + 
	       bBuffer[0] * 0x10000 + bBuffer[1] * 0x1000000;
	   itoa(HexNumber, Lang_Charset_String, 16); 
	   memset(FinalString, '\0', sizeof(char)*10);
	   
	   /* Add leading 0's */
	   index = 0;
	   while (index != (8-strlen(Lang_Charset_String))) {
	       FinalString[index] = '0';
	       index++;
	   }
	   strcat(FinalString, Lang_Charset_String);
	   strcpy(VersionString, "\\StringFileInfo\\");
	   strcat(VersionString, FinalString);
	   strcat(VersionString, "\\ProductVersion");
	 
	   /* Get the product version from the character set info */
	   bRetCode = VerQueryValue((LPVOID)lptstrVffInfo,
	       VersionString, (LPVOID*)&lpVersion,
	       puVersionLen);


	   if (puVersionLen)
	      free(puVersionLen);

	   if (bRetCode) {
	       if (strlen((char*)lpVersion) > SizeVers) {
		  if (hMem)
		     GlobalFree(hMem);              
		  return E_BADSTRINGSIZE;
	      }
	      strcpy(DriverVers, lpVersion);
	      if (hMem)
		 GlobalFree(hMem);              

	   }
	   else {
	       if (hMem)
		  GlobalFree(hMem);              
	       if (strlen(DriverVers) > SizeVers) {
		    return E_BADSTRINGSIZE;
	       }
	       strcpy(DriverVers, "unknown");
	   }
	}
    return E_NONE;
    }
}


short SWGetScreenSize(long *screenWidth, long *screenHeight,
		      long *windowWidth, long *windowHeight)

/* SolidWorks specific function for getting the size of the
   window and the desktop
*/
{
    HWND hWnd;
    RECT rect;

    hWnd = GetDesktopWindow();
    if (hWnd) {
	GetWindowRect(hWnd, &rect);
	*screenWidth = (rect.right - rect.left);
	*screenHeight = (rect.bottom - rect.top);
    }
	
    hWnd = FindWindowEx(NULL, NULL, NULL, "SolidWorks 98");
    SetWindowPos(hWnd, HWND_NOTOPMOST, 0, 0, 1024, 768, 0);

    if (hWnd) {
	GetWindowRect(hWnd, &rect);
	*windowWidth = (rect.right - rect.left);
	*windowHeight = (rect.bottom - rect.top);
    }
    else {
	*windowWidth = 0;
	*windowHeight = 0;
	return E_BADRETURNFUNCTIONCALL;
    }

    return E_NONE;
}

short SWCreateGLWindow( char *GLInfo )
{

    auxInitDisplayMode(AUX_SINGLE|AUX_RGBA);
    auxInitPosition(0, 0, 5, 5);
    auxInitWindow("GLINFO");
    return E_NONE;
}

short SWDestroyGLWindow()
{
    auxQuit();
    return E_NONE;
}

void SWRunReg()
{
    int answer;
    int NT;
    DWORD version;
    SECURITY_ATTRIBUTES secattr;
    STARTUPINFO sInfo;
    PROCESS_INFORMATION  pInfo;
    DWORD exitCode;
    HWND hWnd;
    char SystemDir[256];
    char Executable[512];
    char CommandLine[1024];
    HKEY hkGlobal;
    DWORD dwType,chData=256;
    BYTE DataBuffer[256]; 

    sInfo.cb = sizeof(STARTUPINFO);
    sInfo.lpReserved = NULL;
    sInfo.lpDesktop = NULL; 
    sInfo.lpTitle = NULL; 
    sInfo.dwX = 0; 
    sInfo.dwY = 0; 
    sInfo.dwXSize = 0; 
    sInfo.dwYSize = 0; 
    sInfo.dwXCountChars = 0; 
    sInfo.dwYCountChars = 0; 
    sInfo.dwFillAttribute = 0; 
    sInfo.dwFlags = 0; 
    sInfo.wShowWindow = 0; 
    sInfo.cbReserved2 = 0; 
    sInfo.lpReserved2 = 0; 
    sInfo.hStdInput = 0; 
    sInfo.hStdOutput = 0; 
    sInfo.hStdError = 0; 
    secattr.nLength = sizeof(SECURITY_ATTRIBUTES);
    secattr.lpSecurityDescriptor = NULL;
    secattr.bInheritHandle = FALSE;


    SetLastError(0);
    memset(Executable, '\0', sizeof(char)*512); 
    GetWindowsDirectory(SystemDir, 256);
    strcpy(Executable, SystemDir);
    strcat(Executable, "\\regedit.exe");
    strcpy(CommandLine, Executable);
    version = GetVersion();

    if (version & 0x80000000)
	NT = 0;  
    else
	NT = 1;  

    if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE,"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Setup",
	      0, KEY_ALL_ACCESS, &hkGlobal) != ERROR_SUCCESS)

      fprintf(stderr, "could not open registry key \
         HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Setup\n");

    if ( RegQueryValueEx(hkGlobal, "BootDir", NULL, &dwType,
	      DataBuffer, &chData) != ERROR_SUCCESS)
      fprintf(stderr, "could not read registry value \
         HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Setup\\BootDir\n");

    strcat(CommandLine, " ");
    strcat(CommandLine, DataBuffer);    

    if (!NT)
        strcat(CommandLine, "BenchmarkSW\\Exe\\SWbenchmarkWin95.reg");
    else
        strcat(CommandLine, "BenchmarkSW\\Exe\\SWbenchmark.reg");
    answer =  CreateProcess(Executable, CommandLine,
		&secattr, NULL, FALSE, CREATE_DEFAULT_ERROR_MODE, 
		NULL, NULL, &sInfo, &pInfo);

    answer = GetLastError();
  
 /*   hWnd = FindWindowEx(NULL, NULL, NULL, "Registry Editor");
    while (!hWnd) {
	hWnd = FindWindowEx(NULL, NULL, NULL, "Registry Editor");
    }
    DestroyWindow(hWnd);*/
    GetExitCodeProcess( pInfo.hProcess, &exitCode);
 //   TerminateProcess(pInfo.hProcess, exitCode);
    while (exitCode == STILL_ACTIVE) {
	GetExitCodeProcess( pInfo.hProcess, &exitCode);
    }

}

